@@ -2,7 +2,7 @@
 // GB_sparse_masker_template:  R = masker (C, M, Z) where R is sparse/hyper
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2023, All Rights Reserved.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2025, All Rights Reserved.
 // SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
@@ -47,21 +47,20 @@
 
 #undef GB_COPY_Z
 #if defined ( GB_PHASE_1_OF_2 )
-    #define GB_COPY_Z                                           \
+    #define GB_COPY_Z(i)                                        \
     {                                                           \
         rjnz++ ;                                                \
     }
 #elif defined ( GB_ISO_MASKER )
-    #define GB_COPY_Z                                           \
+    #define GB_COPY_Z(i)                                        \
     {                                                           \
-        Ri [pR] = i ;                                           \
+        GB_ISET (Ri, pR, i) ;                                   \
         pR++ ;                                                  \
     }
 #else
-    #define GB_COPY_Z                                           \
+    #define GB_COPY_Z(i)                                        \
     {                                                           \
-        Ri [pR] = i ;                                           \
-     /* memcpy (Rx +(pR)*rsize, Zx +(Z_iso ? 0:(pZ)*rsize), rsize) ; */     \
+        GB_ISET (Ri, pR, i) ;                                   \
         GB_COPY_Z_TO_R (Rx, pR, Zx, pZ, Z_iso, rsize) ;         \
         pR++ ;                                                  \
     }
@@ -75,15 +74,15 @@
 #if defined ( GB_PHASE_1_OF_2 )
     #define GB_COPY_Z_BITMAP_OR_FULL                            \
     {                                                           \
-        rjnz += GBB_Z (Zb, pZ_start + i - iZ_first) ;             \
+        rjnz += GBb_Z (Zb, pZ_start + i - iZ_first) ;           \
     }
 #elif defined ( GB_ISO_MASKER )
     #define GB_COPY_Z_BITMAP_OR_FULL                            \
     {                                                           \
         int64_t pZ = pZ_start + i - iZ_first ;                  \
-        if (GBB_Z (Zb, pZ))                                       \
+        if (GBb_Z (Zb, pZ))                                     \
         {                                                       \
-            Ri [pR] = i ;                                       \
+            GB_ISET (Ri, pR, i) ;                               \
             pR++ ;                                              \
         }                                                       \
     }
@@ -91,11 +90,10 @@
     #define GB_COPY_Z_BITMAP_OR_FULL                            \
     {                                                           \
         int64_t pZ = pZ_start + i - iZ_first ;                  \
-        if (GBB_Z (Zb, pZ))                                       \
+        if (GBb_Z (Zb, pZ))                                     \
         {                                                       \
-            Ri [pR] = i ;                                       \
-        /*  memcpy (Rx +(pR)*rsize, Zx +(Z_iso ? 0:(pZ)*rsize), rsize) ; */ \
-            GB_COPY_Z_TO_R (Rx, pR, Zx, pZ, Z_iso, rsize) ;         \
+            GB_ISET (Ri, pR, i) ;                               \
+            GB_COPY_Z_TO_R (Rx, pR, Zx, pZ, Z_iso, rsize) ;     \
             pR++ ;                                              \
         }                                                       \
     }
@@ -107,21 +105,20 @@
 
 #undef GB_COPY_C
 #if defined ( GB_PHASE_1_OF_2 )
-    #define GB_COPY_C                                           \
+    #define GB_COPY_C(i)                                        \
     {                                                           \
         rjnz++ ;                                                \
     }
 #elif defined ( GB_ISO_MASKER )
-    #define GB_COPY_C                                           \
+    #define GB_COPY_C(i)                                        \
     {                                                           \
-        Ri [pR] = i ;                                           \
+        GB_ISET (Ri, pR, i) ;                                   \
         pR++ ;                                                  \
     }
 #else
-    #define GB_COPY_C                                           \
+    #define GB_COPY_C(i)                                        \
     {                                                           \
-        Ri [pR] = i ;                                           \
-     /* memcpy (Rx +(pR)*rsize, Cx +(C_iso ? 0:(pC)*rsize), rsize) ; */     \
+        GB_ISET (Ri, pR, i) ;                                   \
         GB_COPY_C_TO_R (Rx, pR, Cx, pC, C_iso, rsize) ;         \
         pR++ ;                                                  \
     }
@@ -138,7 +135,7 @@
     // phase2: compute C
     //--------------------------------------------------------------------------
 
-    ASSERT (GB_C_IS_SPARSE || GB_C_IS_HYPER) ;
+    ASSERT (GB_IS_SPARSE (C) || GB_IS_HYPERSPARSE (C)) ;
 
     #pragma omp parallel for num_threads(R_nthreads) schedule(dynamic,1)
     for (taskid = 0 ; taskid < R_ntasks ; taskid++)
@@ -175,7 +172,7 @@
             // get j, the kth vector of R
             //------------------------------------------------------------------
 
-            int64_t j = GBH_R (Rh, k) ;
+            int64_t j = GBh_R (Rh, k) ;
 
             #if defined ( GB_PHASE_1_OF_2 )
             int64_t rjnz = 0 ;
@@ -186,13 +183,15 @@
                 // A fine task computes a slice of R(:,j)
                 pR     = TaskList [taskid  ].pC ;
                 pR_end = TaskList [taskid+1].pC ;
-                ASSERT (Rp [k] <= pR && pR <= pR_end && pR_end <= Rp [k+1]) ;
+                ASSERT (GB_IGET (Rp, k) <= pR) ;
+                ASSERT (pR <= pR_end) ;
+                ASSERT (pR_end <= GB_IGET (Rp, k+1)) ;
             }
             else
             { 
                 // The vectors of R are never sliced for a coarse task.
-                pR     = Rp [k] ;
-                pR_end = Rp [k+1] ;
+                pR     = GB_IGET (Rp, k) ;
+                pR_end = GB_IGET (Rp, k+1) ;
             }
             int64_t rjnz = pR_end - pR ;
             if (rjnz == 0)
@@ -219,8 +218,8 @@
                 int64_t kC = (R_to_C == NULL) ? j : R_to_C [k] ;
                 if (kC >= 0)
                 { 
-                    pC     = Cp [kC] ;
-                    pC_end = Cp [kC+1] ;
+                    pC     = GB_IGET (Cp, kC) ;
+                    pC_end = GB_IGET (Cp, kC+1) ;
                 }
             }
 
@@ -230,12 +229,12 @@
             #if defined ( GB_PHASE_2_OF_2 ) || defined ( GB_DEBUG )
             // get the first index in C(:,j) for this vector
             int64_t iC_first = -1 ;
-            if (cjnz > 0) iC_first = Ci [pC] ;
+            if (cjnz > 0) iC_first = GB_IGET (Ci, pC) ;
             #endif
 
             #ifdef GB_DEBUG
             int64_t iC_last = -1 ;
-            if (cjnz > 0) iC_last  = Ci [pC_end-1] ;
+            if (cjnz > 0) iC_last  = GB_IGET (Ci, pC_end-1) ;
             #endif
 
             //------------------------------------------------------------------
@@ -256,8 +255,8 @@
                 int64_t kZ = (R_to_Z == NULL) ? j : R_to_Z [k] ;
                 if (kZ >= 0)
                 { 
-                    pZ     = GBP_Z (Zp, kZ, vlen) ;
-                    pZ_end = GBP_Z (Zp, kZ+1, vlen) ;
+                    pZ     = GBp_Z (Zp, kZ, vlen) ;
+                    pZ_end = GBp_Z (Zp, kZ+1, vlen) ;
                 }
             }
 
@@ -265,11 +264,16 @@
             int64_t pZ_start = pZ ;
             bool zdense = (zjnz == len) && (zjnz > 0) ;
 
-            int64_t iZ_first = -1, iZ_last = -1 ;
+            int64_t iZ_first = -1 ;
+            #ifdef GB_DEBUG
+            int64_t iZ_last = -1 ;
+            #endif
             if (zjnz > 0)
             {
-                iZ_first = GBI_Z (Zi, pZ, vlen) ;
-                iZ_last  = GBI_Z (Zi, pZ_end-1, vlen) ;
+                iZ_first = GBi_Z (Zi, pZ, vlen) ;
+                #ifdef GB_DEBUG
+                iZ_last  = GBi_Z (Zi, pZ_end-1, vlen) ;
+                #endif
             }
 
             //------------------------------------------------------------------
@@ -290,8 +294,8 @@
                 int64_t kM = (R_to_M == NULL) ? j : R_to_M [k] ;
                 if (kM >= 0)
                 { 
-                    pM     = GBP_M (Mp, kM, vlen) ;
-                    pM_end = GBP_M (Mp, kM+1, vlen) ;
+                    pM     = GBp_M (Mp, kM, vlen) ;
+                    pM_end = GBp_M (Mp, kM+1, vlen) ;
                 }
             }
 
@@ -301,7 +305,7 @@
             // get the first index in M(:,j) for this vector
             int64_t iM_first = -1 ;
             int64_t pM_first = pM ;
-            if (mjnz > 0) iM_first = GBI_M (Mi, pM_first, vlen) ;
+            if (mjnz > 0) iM_first = GBi_M (Mi, pM_first, vlen) ;
 
             //------------------------------------------------------------------
             // R(:,j) = masker (C (:,j), M (:,j), Z (:,j))
@@ -325,7 +329,7 @@
                 // Otherwise, R is bitmap and not computed here, but in
                 // GB_bitmap_masker_template instead.
 
-                ASSERT (GB_M_IS_SPARSE || GB_M_IS_HYPER) ;
+                ASSERT (GB_IS_SPARSE (M) || GB_IS_HYPERSPARSE (M)) ;
                 ASSERT (!GB_MASK_COMP) ;
 
                 // 2-way merge of C(:,j) and M(:,j) and direct lookup of Z
@@ -333,15 +337,14 @@
                 while (pC < pC_end && pM < pM_end)
                 {
                     
-                    int64_t iC = Ci [pC] ;
-                    int64_t iM = Mi [pM] ;
+                    int64_t iC = GB_IGET (Ci, pC) ;
+                    int64_t iM = GB_IGET (Mi, pM) ;
 
                     if (iC < iM)
                     { 
                         // C(i,j) is present but M(i,j) is not
                         // R(i,j) = C(i,j)
-                        int64_t i = iC ;
-                        GB_COPY_C ;
+                        GB_COPY_C (iC) ;
                         pC++ ;
                     }
                     else if (iC > iM)
@@ -369,7 +372,7 @@
                         else
                         { 
                             // R(i,j) = C(i,j)
-                            GB_COPY_C ;
+                            GB_COPY_C (i) ;
                         }
                         pC++ ;
                         pM++ ;
@@ -383,8 +386,8 @@
                 for ( ; pC < pC_end ; pC++)
                 { 
                     // C(i,j) is present but M(i,j) is not
-                    int64_t i = Ci [pC] ;
-                    GB_COPY_C ;
+                    int64_t i = GB_IGET (Ci, pC) ;
+                    GB_COPY_C (i) ;
                 }
                 #endif
 
@@ -392,7 +395,7 @@
                 for ( ; pM < pM_end ; pM++)
                 {
                     // M(i,j) is present but C(i,j) is not
-                    int64_t i = Mi [pM] ;
+                    int64_t i = GB_IGET (Mi, pM) ;
                     bool mij = GB_MCAST (Mx, pM, msize) ;
                     if (mij)
                     { 
@@ -422,7 +425,7 @@
                 //      sparse  sparse      sparse          sparse
 
                 // Z must be sparse or hypersparse
-                ASSERT (GB_Z_IS_SPARSE || GB_Z_IS_HYPER) ;
+                ASSERT (GB_IS_SPARSE (Z) || GB_IS_HYPERSPARSE (Z)) ;
 
                 if (!GB_MASK_COMP)
                 { 
@@ -436,20 +439,13 @@
                     rjnz = cjnz ;
                     #else
                     ASSERT (rjnz == cjnz) ;
-                    memcpy (Ri +(pR), Ci +(pC), cjnz * sizeof (int64_t)) ;
+                    for (int64_t k = 0 ; k < cjnz ; k++)
+                    { 
+                        int64_t i = GB_IGET (Ci, pC + k) ;
+                        GB_ISET (Ri, pR + k, i) ;
+                    }
                     #ifndef GB_ISO_MASKER
                     GB_COPY_C_TO_R_RANGE (Rx, pR, Cx, pC, C_iso, rsize, cjnz) ;
-//                  if (C_iso)
-//                  {
-//                      for (int64_t k = 0 ; k < cjnz ; k++)
-//                      {
-//                          memcpy (Rx +(pR+k)*rsize, Cx, rsize) ;
-//                      }
-//                  }
-//                  else
-//                  {
-//                      memcpy (Rx +(pR)*rsize, Cx +(pC)*rsize, cjnz*rsize) ;
-//                  }
                     #endif
                     #endif
 
@@ -466,21 +462,13 @@
                     rjnz = zjnz ;
                     #else
                     ASSERT (rjnz == zjnz) ;
-                    memcpy (Ri +(pR), Zi +(pZ), zjnz * sizeof (int64_t)) ;
+                    for (int64_t k = 0 ; k < zjnz ; k++)
+                    { 
+                        int64_t i = GB_IGET (Zi, pZ + k) ;
+                        GB_ISET (Ri, pR + k, i) ;
+                    }
                     #ifndef GB_ISO_MASKER
                     GB_COPY_Z_TO_R_RANGE (Rx, pR, Zx, pZ, Z_iso, rsize, zjnz) ;
-//                  if (Z_iso)
-//                  {
-//                      for (int64_t k = 0 ; k < zjnz ; k++)
-//                      {
-//                          memcpy (Rx +(pR+k)*rsize, Zx, rsize) ;
-//                      }
-//                  }
-//                  else
-//                  {
-//                      memcpy (Rx +(pR)*rsize, Zx +(pZ)*rsize, zjnz*rsize) ;
-//                  }
-
                     #endif
                     #endif
                 }
@@ -513,7 +501,7 @@
                 // present), but both C and Z are stored in a sparse or
                 // hypersparse sparsity structure.  M has any sparsity.
 
-                ASSERT (GB_Z_IS_SPARSE || GB_Z_IS_HYPER) ;
+                ASSERT (GB_IS_SPARSE (Z) || GB_IS_HYPERSPARSE (Z)) ;
 
                 ASSERT (cjnz == zjnz) ;
                 ASSERT (iC_first == iZ_first) ;
@@ -525,12 +513,12 @@
                 for (int64_t p = 0 ; p < cjnz ; p++)
                 {
                     int64_t i = p + iC_first ;
-                    Ri [pR + p] = i ;
-                    int64_t iM = (pM < pM_end) ? GBI_M (Mi,pM,vlen) : INT64_MAX;
+                    GB_ISET (Ri, pR + p, i) ;   // Ri [pR + p] = i ;
+                    int64_t iM = (pM < pM_end) ? GBi_M (Mi,pM,vlen) : INT64_MAX;
                     bool mij = false ;
                     if (i == iM)
                     { 
-                        mij = GBB_M (Mb, pM) && GB_MCAST (Mx, pM, msize) ;
+                        mij = GBb_M (Mb, pM) && GB_MCAST (Mx, pM, msize) ;
                         pM++ ;
                     }
                     if (GB_MASK_COMP) mij = !mij ;
@@ -538,15 +526,11 @@
                     if (mij)
                     { 
                         // R(i,j) = Z (i,j)
-//                      memcpy (Rx +(pR+p)*rsize, Zx +(Z_iso? 0:(pZ+p)*rsize),
-//                          rsize) ;
                         GB_COPY_Z_TO_R (Rx, pR+p, Zx, pZ+p, Z_iso, rsize) ;
                     }
                     else
                     { 
                         // R(i,j) = C (i,j)
-//                      memcpy (Rx +(pR+p)*rsize, Cx +(C_iso? 0:(pC+p)*rsize),
-//                          rsize) ;
                         GB_COPY_C_TO_R (Rx, pR+p, Cx, pC+p, C_iso, rsize) ;
                     }
                     #endif
@@ -562,7 +546,7 @@
                 //--------------------------------------------------------------
 
                 // Z is sparse or hypersparse; M has any sparsity structure
-                ASSERT (GB_Z_IS_SPARSE || GB_Z_IS_HYPER) ;
+                ASSERT (GB_IS_SPARSE (Z) || GB_IS_HYPERSPARSE (Z)) ;
 
                 //--------------------------------------------------------------
                 // Z is sparse or hypersparse, M has any sparsity
@@ -591,8 +575,8 @@
                     // get the next i for R(:,j)
                     //----------------------------------------------------------
 
-                    int64_t iC = Ci [pC] ;
-                    int64_t iZ = Zi [pZ] ;
+                    int64_t iC = GB_IGET (Ci, pC) ;
+                    int64_t iZ = GB_IGET (Zi, pZ) ;
                     int64_t i = GB_IMIN (iC, iZ) ;
 
                     //----------------------------------------------------------
@@ -615,8 +599,8 @@
                         // let pM = pM_first + delta
                         // then delta = i - iM_first
                         pM = pM_first + (i - iM_first) ;
-                        ASSERT (i == GBI_M (Mi, pM, vlen)) ;
-                        mij = GBB_M (Mb, pM) && GB_MCAST (Mx, pM, msize) ;
+                        ASSERT (i == GBi_M (Mi, pM, vlen)) ;
+                        mij = GBb_M (Mb, pM) && GB_MCAST (Mx, pM, msize) ;
                         // increment pM for the wrapup phase below
                         pM++ ;
 
@@ -628,15 +612,16 @@
                         // Method04b: M(:,j) is sparse
                         //------------------------------------------------------
 
-                        // Use GB_SPLIT_BINARY_SEARCH so that pM can be used in
+                        // Use GB_split_binary_search so that pM can be used in
                         // the for loop with index pM in the wrapup phase.
-                        ASSERT (GB_M_IS_SPARSE || GB_M_IS_HYPER) ;
+                        ASSERT (GB_IS_SPARSE (M) || GB_IS_HYPERSPARSE (M)) ;
                         int64_t pright = pM_end - 1 ;
                         bool found ;
-                        GB_SPLIT_BINARY_SEARCH (i, Mi, pM, pright, found) ;
+                        found = GB_split_binary_search (i, Mi, GB_Mi_IS_32,
+                            &pM, &pright) ;
                         if (found)
                         { 
-                            ASSERT (i == Mi [pM]) ;
+                            ASSERT (i == GB_IGET (Mi, pM)) ;
                             mij = GB_MCAST (Mx, pM, msize) ;
                             // increment pM for the wrapup phase below
                             pM++ ;
@@ -652,26 +637,25 @@
                     if (iC < iZ)
                     { 
                         // C(i,j) is present but Z(i,j) is not
-                        if (!mij) GB_COPY_C ;
+                        if (!mij) GB_COPY_C (i) ;
                         pC++ ;
                     }
                     else if (iC > iZ)
                     { 
                         // Z(i,j) is present but C(i,j) is not
-                        if (mij) GB_COPY_Z ;
+                        if (mij) GB_COPY_Z (i) ;
                         pZ++ ;
                     }
                     else
                     {
                         // both C(i,j) and Z(i,j) are present
-                        int64_t i = iC ;
                         if (mij)
                         { 
-                            GB_COPY_Z ;
+                            GB_COPY_Z (iC) ;
                         }
                         else
                         { 
-                            GB_COPY_C ;
+                            GB_COPY_C (iC) ;
                         }
                         pC++ ;
                         pZ++ ;
@@ -709,13 +693,13 @@
 
                             for ( ; pZ < pZ_end ; pZ++)
                             { 
-                                int64_t i = Zi [pZ] ;
+                                int64_t i = GB_IGET (Zi, pZ) ;
                                 // mask is dense, lookup M(i,j)
                                 pM = pM_first + (i - iM_first) ;
-                                ASSERT (i == GBI_M (Mi, pM, vlen)) ;
-                                bool mij = GBB_M (Mb, pM) &&
+                                ASSERT (i == GBi_M (Mi, pM, vlen)) ;
+                                bool mij = GBb_M (Mb, pM) &&
                                            GB_MCAST (Mx, pM, msize) ;
-                                if (mij) GB_COPY_Z ;
+                                if (mij) GB_COPY_Z (i) ;
                             }
 
                         }
@@ -729,16 +713,17 @@
                             // This loop requires pM to start at the first
                             // entry in M(:,j) that has not yet been handled.
 
-                            ASSERT (GB_M_IS_SPARSE || GB_M_IS_HYPER) ;
+                            ASSERT (GB_IS_SPARSE (M) || GB_IS_HYPERSPARSE (M)) ;
                             for ( ; pM < pM_end ; pM++)
                             {
                                 if (GB_MCAST (Mx, pM, msize))
                                 { 
-                                    int64_t i = Mi [pM] ;
+                                    int64_t i = GB_IGET (Mi, pM) ;
                                     int64_t pright = pZ_end - 1 ;
                                     bool found ;
-                                    GB_BINARY_SEARCH (i, Zi, pZ, pright, found);
-                                    if (found) GB_COPY_Z ;
+                                    found = GB_binary_search (i, Zi,GB_Zi_IS_32,
+                                        &pZ, &pright) ;
+                                    if (found) GB_COPY_Z (i) ;
                                 }
                             }
 
@@ -750,18 +735,19 @@
                             // Method04e: M(:,j) is much denser than Z(:,j)
                             //--------------------------------------------------
 
-                            ASSERT (GB_M_IS_SPARSE || GB_M_IS_HYPER) ;
+                            ASSERT (GB_IS_SPARSE (M) || GB_IS_HYPERSPARSE (M)) ;
                             for ( ; pZ < pZ_end ; pZ++)
                             { 
                                 // wrapup, C now empty, M(:,j) much denser than
                                 // Z(:,j); C, M and Z all sparse
-                                int64_t i = Zi [pZ] ;
+                                int64_t i = GB_IGET (Zi, pZ) ;
                                 bool mij = false ;
                                 int64_t pright = pM_end - 1 ;
                                 bool found ;
-                                GB_BINARY_SEARCH (i, Mi, pM, pright,found) ;
+                                found = GB_binary_search (i, Mi, GB_Mi_IS_32,
+                                    &pM, &pright) ;
                                 if (found) mij = GB_MCAST (Mx, pM, msize) ;
-                                if (mij) GB_COPY_Z ;
+                                if (mij) GB_COPY_Z (i) ;
                             }
 
                         }
@@ -772,11 +758,11 @@
                             // Method04f: M(:,j) and Z(:,j) about same # entries
                             //--------------------------------------------------
 
-                            ASSERT (GB_M_IS_SPARSE || GB_M_IS_HYPER) ;
+                            ASSERT (GB_IS_SPARSE (M) || GB_IS_HYPERSPARSE (M)) ;
                             while (pM < pM_end && pZ < pZ_end)
                             {
-                                int64_t iM = Mi [pM] ;
-                                int64_t i = Zi [pZ] ;
+                                int64_t iM = GB_IGET (Mi, pM) ;
+                                int64_t i = GB_IGET (Zi, pZ) ;
                                 if (iM < i)
                                 { 
                                     // M(i,j) exists but not Z(i,j)
@@ -790,7 +776,7 @@
                                 else
                                 { 
                                     // both M(i,j) and Z(i,j) exist
-                                    if (GB_MCAST (Mx, pM, msize)) GB_COPY_Z ;
+                                    if (GB_MCAST (Mx, pM, msize)) GB_COPY_Z (i);
                                     pM++ ;
                                     pZ++ ;
                                 }
@@ -814,13 +800,13 @@
 
                             for ( ; pZ < pZ_end ; pZ++)
                             { 
-                                int64_t i = Zi [pZ] ;
+                                int64_t i = GB_IGET (Zi, pZ) ;
                                 // mask is dense, lookup M(i,j)
                                 pM = pM_first + (i - iM_first) ;
-                                ASSERT (i == GBI_M (Mi, pM, vlen)) ;
-                                bool mij = GBB_M (Mb, pM) &&
+                                ASSERT (i == GBi_M (Mi, pM, vlen)) ;
+                                bool mij = GBb_M (Mb, pM) &&
                                            GB_MCAST (Mx, pM, msize) ;
-                                if (!mij) GB_COPY_Z ;   // mask is complemented
+                                if (!mij) GB_COPY_Z (i) ; // mask complemented
                             }
                         }
                         else
@@ -830,16 +816,17 @@
                             // Method04h: M(:,j) is sparse
                             //--------------------------------------------------
 
-                            ASSERT (GB_M_IS_SPARSE || GB_M_IS_HYPER) ;
+                            ASSERT (GB_IS_SPARSE (M) || GB_IS_HYPERSPARSE (M)) ;
                             for ( ; pZ < pZ_end ; pZ++)
                             { 
-                                int64_t i = Zi [pZ] ;
+                                int64_t i = GB_IGET (Zi, pZ) ;
                                 bool mij = false ;
                                 int64_t pright = pM_end - 1 ;
                                 bool found ;
-                                GB_BINARY_SEARCH (i, Mi, pM, pright, found) ;
+                                found = GB_binary_search (i, Mi, GB_Mi_IS_32,
+                                    &pM, &pright) ;
                                 if (found) mij = GB_MCAST (Mx, pM, msize) ;
-                                if (!mij) GB_COPY_Z ;   // mask is complemented
+                                if (!mij) GB_COPY_Z (i) ; // mask complemented
                             }
                         }
                     }
@@ -868,13 +855,13 @@
 
                             for ( ; pC < pC_end ; pC++)
                             { 
-                                int64_t i = Ci [pC] ;
+                                int64_t i = GB_IGET (Ci, pC) ;
                                 // mask is dense, lookup M(i,j)
                                 pM = pM_first + (i - iM_first) ;
-                                ASSERT (i == GBI_M (Mi, pM, vlen)) ;
-                                bool mij = GBB_M (Mb, pM) &&
+                                ASSERT (i == GBi_M (Mi, pM, vlen)) ;
+                                bool mij = GBb_M (Mb, pM) &&
                                            GB_MCAST (Mx, pM, msize) ;
-                                if (mij) GB_COPY_C ;
+                                if (mij) GB_COPY_C (i) ;
                             }
 
                         }
@@ -885,16 +872,17 @@
                             // Method04j: C(:,j) is much denser than M(:,j)
                             //--------------------------------------------------
 
-                            ASSERT (GB_M_IS_SPARSE || GB_M_IS_HYPER) ;
+                            ASSERT (GB_IS_SPARSE (M) || GB_IS_HYPERSPARSE (M)) ;
                             for ( ; pM < pM_end ; pM++)
                             {
                                 if (GB_MCAST (Mx, pM, msize))
                                 { 
-                                    int64_t i = Mi [pM] ;
+                                    int64_t i = GB_IGET (Mi, pM) ;
                                     int64_t pright = pC_end - 1 ;
                                     bool found ;
-                                    GB_BINARY_SEARCH (i, Ci, pC, pright, found);
-                                    if (found) GB_COPY_C ;
+                                    found = GB_binary_search (i, Ci,GB_Ci_IS_32,
+                                        &pC, &pright) ;
+                                    if (found) GB_COPY_C (i) ;
                                 }
                             }
 
@@ -906,16 +894,17 @@
                             // Method04k: M(:,j) is much denser than C(:,j)
                             //--------------------------------------------------
 
-                            ASSERT (GB_M_IS_SPARSE || GB_M_IS_HYPER) ;
+                            ASSERT (GB_IS_SPARSE (M) || GB_IS_HYPERSPARSE (M)) ;
                             for ( ; pC < pC_end ; pC++)
                             { 
-                                int64_t i = Ci [pC] ;
+                                int64_t i = GB_IGET (Ci, pC) ;
                                 bool mij = false ;
                                 int64_t pright = pM_end - 1 ;
                                 bool found ;
-                                GB_BINARY_SEARCH (i, Mi, pM, pright, found);
+                                found = GB_binary_search (i, Mi, GB_Mi_IS_32,
+                                    &pM, &pright) ;
                                 if (found) mij = GB_MCAST (Mx, pM, msize) ;
-                                if (mij) GB_COPY_C ;
+                                if (mij) GB_COPY_C (i) ;
                             }
 
                         }
@@ -926,11 +915,11 @@
                             // Method04l: M(:,j) and C(:,j) about same # entries
                             //--------------------------------------------------
 
-                            ASSERT (GB_M_IS_SPARSE || GB_M_IS_HYPER) ;
+                            ASSERT (GB_IS_SPARSE (M) || GB_IS_HYPERSPARSE (M)) ;
                             while (pM < pM_end && pC < pC_end)
                             {
-                                int64_t iM = Mi [pM] ;
-                                int64_t i = Ci [pC] ;
+                                int64_t iM = GB_IGET (Mi, pM) ;
+                                int64_t i = GB_IGET (Ci, pC) ;
                                 if (iM < i)
                                 { 
                                     // M(i,j) exists but not C(i,j)
@@ -944,7 +933,7 @@
                                 else
                                 { 
                                     // both M(i,j) and C(i,j) exist
-                                    if (GB_MCAST (Mx, pM, msize)) GB_COPY_C ;
+                                    if (GB_MCAST (Mx, pM, msize)) GB_COPY_C (i);
                                     pM++ ;
                                     pC++ ;
                                 }
@@ -968,13 +957,13 @@
 
                             for ( ; pC < pC_end ; pC++)
                             { 
-                                int64_t i = Ci [pC] ;
+                                int64_t i = GB_IGET (Ci, pC) ;
                                 // mask is dense, lookup M(i,j)
                                 pM = pM_first + (i - iM_first) ;
-                                ASSERT (i == GBI_M (Mi, pM, vlen)) ;
-                                bool mij = GBB_M (Mb, pM) &&
+                                ASSERT (i == GBi_M (Mi, pM, vlen)) ;
+                                bool mij = GBb_M (Mb, pM) &&
                                            GB_MCAST (Mx, pM, msize) ;
-                                if (!mij) GB_COPY_C ;
+                                if (!mij) GB_COPY_C (i) ;
                             }
                         }
                         else
@@ -984,17 +973,18 @@
                             // Method04n: M(:,j) is sparse
                             //--------------------------------------------------
 
-                            ASSERT (GB_M_IS_SPARSE || GB_M_IS_HYPER) ;
+                            ASSERT (GB_IS_SPARSE (M) || GB_IS_HYPERSPARSE (M)) ;
                             for ( ; pC < pC_end ; pC++)
                             { 
-                                int64_t i = Ci [pC] ;
+                                int64_t i = GB_IGET (Ci, pC) ;
                                 // M(i,j) false if not present
                                 bool mij = false ; 
                                 int64_t pright = pM_end - 1 ;
                                 bool found ;
-                                GB_BINARY_SEARCH (i, Mi, pM, pright, found) ;
+                                found = GB_binary_search (i, Mi, GB_Mi_IS_32,
+                                    &pM, &pright) ;
                                 if (found) mij = GB_MCAST (Mx, pM, msize) ;
-                                if (!mij) GB_COPY_C ;
+                                if (!mij) GB_COPY_C (i) ;
                             }
                         }
                     }
@@ -1016,7 +1006,7 @@
             }
             else
             { 
-                Rp [k] = rjnz ;
+                GB_ISET (Rp, k, rjnz) ; // Rp [k] = rjnz ;
             }
             #endif
         }
